@@ -14,7 +14,7 @@ import cmdstanpy
 
 __author__ = """Johannes Buchner"""
 __email__ = 'johannes.buchner.acad@gmx.com'
-__version__ = '1.1.1'
+__version__ = '1.2.0'
 
 path = os.path.expanduser("~/.stan_cache")
 mem = joblib.Memory(path, verbose=False)
@@ -106,6 +106,7 @@ def hash_data(datafile):
 
 	return md5.hexdigest()
 
+
 def get_formatted_code(code):
 	"""Get reasonably readable formatted code from trimmed code.
 
@@ -123,13 +124,13 @@ def get_formatted_code(code):
 	indent = 0
 	for i, line in enumerate(code.split("\n")):
 		indent -= line.count('}')
-		formatted_code_lines.append('%3d: %s%s' % (i+1, '  ' * indent, line))
+		formatted_code_lines.append('%3d: %s%s' % (i + 1, '  ' * indent, line))
 		indent += line.count('{')
 	return '\n'.join(formatted_code_lines)
 
 
-@mem.cache
-def cached_run_stan(code, datafile, **kwargs):
+@mem.cache(ignore=['verbose'])
+def cached_run_stan(code, datafile, verbose=True, **kwargs):
 	"""Run MCMC with the given code and data file.
 
 	Parameters
@@ -138,6 +139,8 @@ def cached_run_stan(code, datafile, **kwargs):
 		Stan model code
 	datafile: str
 		Path to data file
+	verbose: bool
+		whether to print the code being compiled, posterior summaries and diagnostics.
 	**kwargs: dict
 		arguments passed on to `cmdstanpy.CmdStanModel.sample`
 
@@ -148,9 +151,10 @@ def cached_run_stan(code, datafile, **kwargs):
 	method_variables: object
 		method_variables returned by fit object
 	"""
-	print("Code")
-	print("----")
-	print(get_formatted_code(code))
+	if verbose:
+		print("Code")
+		print("----")
+		print(get_formatted_code(code))
 	code_hash = hash_model_code(code)
 	codefile = os.path.join(path, code_hash + '.stan')
 	if not os.path.exists(codefile):
@@ -160,17 +164,18 @@ def cached_run_stan(code, datafile, **kwargs):
 	assert model.code() == code, (model.code, code)
 
 	fit = model.sample(data=datafile, **kwargs)
-	print("Summary")
-	print("-------")
-	print(fit.summary())
-	print("Diagnostics")
-	print("-----------")
-	print(fit.diagnose())
+	if verbose:
+		print("Summary")
+		print("-------")
+		print(fit.summary())
+		print("Diagnostics")
+		print("-----------")
+		print(fit.diagnose())
 
 	return fit.stan_variables(), fit.method_variables()
 
 
-def run_stan(code, data, **kwargs):
+def run_stan(code, data, verbose=True, **kwargs):
 	"""Run MCMC with the given code and data.
 
 	Parameters
@@ -179,6 +184,9 @@ def run_stan(code, data, **kwargs):
 		Stan model code
 	data: dict
 		Model data
+	verbose: bool
+		whether to print the code being compiled, summaries of the
+		input data and posterior, and convergence diagnostics.
 	**kwargs: dict
 		arguments passed on to `cmdstanpy.CmdStanModel.sample`
 
@@ -191,29 +199,39 @@ def run_stan(code, data, **kwargs):
 	"""
 	simple_model_code = trim_model_code(code)
 
-	print()
-	print("Data")
-	print("----")
-	for k, v in data.items():
-		shape = np.shape(v)
-		if shape == ():
-			print('  %-10s: %s' % (k, v))
-		elif shape[0] == 0:
-			print('  %-10s: shape %s' % (k, shape))
-		else:
-			print('  %-10s: shape %s [%s ... %s]' % (k, shape, np.min(v), np.max(v)))
-		del k, v
+	if verbose:
+		print()
+		print("Data")
+		print("----")
+		for k, v in data.items():
+			shape = np.shape(v)
+			if shape == ():
+				print('  %-10s: %s' % (k, v))
+			elif shape[0] == 0:
+				print('  %-10s: shape %s' % (k, shape))
+			else:
+				print('  %-10s: shape %s [%s ... %s]' % (k, shape, np.min(v), np.max(v)))
+			del k, v
 
-	data2 = {k: data[k] for k in sorted(data.keys())}
-	f = tempfile.NamedTemporaryFile(suffix='.json')
-	fname = f.name
-	cmdstanpy.write_stan_json(fname, data2)
-	data_hash = hash_data(fname)
+	with tempfile.NamedTemporaryFile(suffix='.json') as f:
+		fname = f.name
+		data_sorted = {k: data[k] for k in sorted(data.keys())}
+		cmdstanpy.write_stan_json(fname, data_sorted)
+		del data_sorted
+		data_hash = hash_data(fname)
 
-	datafile = os.path.join(path, data_hash + '.json')
-	shutil.copy(fname, datafile)
+		datafile = os.path.join(path, data_hash + '.json')
+		shutil.copy(fname, datafile)
+
+	results = cached_run_stan(simple_model_code, datafile, verbose=verbose, **kwargs)
+
+	try:
+		os.unlink(datafile)
+	except IOError as e:
+		warnings.warn('Cleaning up stan input data file failed: %s' % e)
+
+	return results
 
-	return cached_run_stan(simple_model_code, datafile, **kwargs)
 
 def remove_stuck_chains(stan_variables, method_variables):
 	"""
@@ -230,8 +248,7 @@ def remove_stuck_chains(stan_variables, method_variables):
 	-------
 	stan_variables: dict
 		Dictionary of variables with their posterior chains.
-	"""	
-	
+	"""
 	# identify the likelihood range of each chain
 	# select the chain with the highest likelihood
 	# include all chains which cross into this likelihood range
@@ -242,19 +259,20 @@ def remove_stuck_chains(stan_variables, method_variables):
 	top_chain_min_lp = lp[:, top_chain_index].min()
 	chain_mask = lp.max(axis=0) > top_chain_min_lp
 	if not chain_mask.all():
-		warnings.warn("Ignoring these stuck chains: %s" % (np.where(~chain_mask)[0]+1))
+		warnings.warn("Ignoring these stuck chains: %s" % (np.where(~chain_mask)[0] + 1))
 
 	chain_length, num_chains = lp.shape
 	mask = np.zeros(lp.size, dtype=bool)
 	for i in np.where(chain_mask)[0]:
-		mask[i * chain_length : (i + 1) * chain_length] = True
+		mask[i * chain_length: (i + 1) * chain_length] = True
 
 	filtered_variables = collections.OrderedDict()
 	for k, v in stan_variables.items():
-		filtered_variables[k] = v[mask,...]
+		filtered_variables[k] = v[mask, ...]
 
 	return filtered_variables
 
+
 def plot_corner(stan_variables, max_array_size=20, verbose=True, **kwargs):
 	"""
 	Make a simple corner plot, based on samples extracted from fit.
